#!/usr/bin/python
# G_waveform_spectrum_file_outputs.py D. Gibbon, 2021-07-06
# You can insert your own code for spectrum file output.

import sys, os, re													# import specialised modules
import numpy as np
import matplotlib.pyplot as plt
import scipy.io.wavfile as wave
from scipy.signal import butter, lfilter

def outputtextlines(text, filename):
	handle = open(filename,'w')
	linelist = handle.write(text)
	handle.close()
	return

def appendtextlines(text, filename):
	handle = open(filename,'a')
	linelist = handle.write(text)
	handle.close()
	return

if len(sys.argv) > 1:
	wavfilename = sys.argv[1]							# get input filename from command line 
else:
	wavfilename = "DATA/one-to-ten-Putonghua-Lara-16k-mono.wav"

fs, signal = wave.read(wavfilename)						# read sampling frequency and signal
wavfilebase = re.sub("^.*/","",wavfilename)
wavfilebase = re.sub("-16k-mono","",wavfilebase[:-4])
figurefilename = "PNG/G_%s.png"%wavfilebase
csvfilename = "CSV/G_%s.csv"%wavfilebase

signallength = len(signal)								# define signal length in bytes
signalseconds = signallength / fs						# define signal length in seconds
signal = signal / max(abs(signal))						# normalise signal -1 ... 0 ... 1

b, a = butter(5, 5 / (0.5 * fs), btype="low")			# define Butterworth filter
envelope = lfilter(b, a, abs(signal))					# apply filter to create lf envelope
envelope = envelope / max(envelope)						# normalise envelope 0 ... 1

specmags = np.abs(np.fft.rfft(envelope))
specmags = specmags / np.max(specmags)
specmaglen = len(specmags)
specfreqs = np.linspace(0,fs/2,specmaglen)

spectrummax = 3
lfspecmaglen = int(round(spectrummax * specmaglen / (fs / 2)))
lfspecmags = specmags[1:lfspecmaglen]
lfspecfreqs = specfreqs[1:lfspecmaglen]

topmagscount = 6										# define max frequency of lf spectrum
topmags = sorted(lfspecmags)[-topmagscount:]
toppos = [ list(lfspecmags).index(m) for m in topmags ]
topfreqs = [ lfspecfreqs[p] for p in toppos ]

csvfreqs = "lffreqs\t"+"\t".join([ "%.3f"%x for x in lfspecfreqs ])+"\n"
csvmags = "lfmags\t"+"\t".join([ "%.3f"%x for x in lfspecmags ])+"\n"
outputtextlines(csvfreqs, csvfilename)
appendtextlines(csvmags, csvfilename)
os.system("soffice %s"%csvfilename)
#-----------------------------------------------------------------------------------------
fig,((plt01, plt02)) = plt.subplots(nrows=1, ncols=2, figsize=(14, 4))	# define figure format
plt.suptitle("%s, %d"%(wavfilename, fs), fontweight="bold")	# display a title

xaxistime = np.linspace(0, signalseconds, signallength)		# define x axis in seconds
plt01.plot(xaxistime, signal, color="lightgrey")			# plot waveform in grey
plt01.plot(xaxistime, envelope, color="red")
plt01.set_xlabel("Time")
plt01.set_ylabel("Amplitude")
plt.xlim(0,np.ceil(signalseconds))

plt02.plot(lfspecfreqs, lfspecmags)
plt02.scatter(topfreqs, topmags, color="red")
for f,m in zip(topfreqs, topmags):
	plt02.text(f, m-0.1, "%.3fHz\n%dms"%(f,1000/f), fontsize=8)
plt02.set_xlabel("Frequency")
plt02.set_ylabel("Magnitude")
plt02.set_xlim(0,spectrummax)

plt.tight_layout(pad=3)
plt.savefig(figurefilename)
plt.show()	
